/*

MetaTrader History Toolkit
example.cpp
v1.0
2013-02-03
Copyright  2013 Heinz Traub

*/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <ctime>
#include "..\..\import\hcc.h"

void pause();
bool CheckError(ReturnCode);

int main(int argc, char* argv[])
{
	printf("MetaTrader History Toolkit - C++ Example\n");
	printf("Copyright (C) 2013 Heinz Traub\n");
	if(argc > 1)
	{
		int nChars = MultiByteToWideChar(CP_UTF8, 0, argv[1], -1, NULL, 0);
		if(nChars > 0)
		{
			const wchar_t* hcc_path = new wchar_t[nChars];
			if(MultiByteToWideChar(CP_UTF8, 0, argv[1], -1, (LPWSTR)hcc_path, nChars) == nChars)
			{
				HCCHandle file;
				ReturnCode rc = HCCOpenFile(hcc_path, &file);
				if(!CheckError(rc))
					return 1;
				wprintf(L"\nFile: %ls\n", hcc_path);
				wchar_t wsymbol[10]; // Should fit any symbol.
				rc = HCCGetSymbol(file, wsymbol);
				if(!CheckError(rc))
					return 1;
				wprintf(L"Symbol: %ls\n", wsymbol);
				printf("Timeframe: 1 Minute\n");
				printf("Loading history...");
				size_t hlength;
				rc = HCCLoadHistory(file, M1, 0, 3, &hlength); // Load only the first 3 rates.
				if(!CheckError(rc))
					return 1;
				printf("\nRates Loaded: %u", hlength);
				if(hlength > 0)
				{
					printf("\nThis program will display the first 3 rates (if available):");
					pause();
					HCCRates r;
					unsigned int precision;
					char price_format[10];
					rc = HCCGetPrecision(file, &precision);
					if(rc == HCC_SUCCESS && precision > 0)
					{
						char pre_str[3];
						strcpy(price_format, "\n%s: %.");
						itoa(precision, pre_str, 10);
						strcat(price_format, pre_str);
						strcat(price_format, "f");
					}
					else
						precision = 0;
					for(size_t i = 0; i < hlength; i++)
					{
						rc = HCCGetRates(file, i, &r);
						if(!CheckError(rc))
							return 1;
						printf("\n\n[RATES #%u]", i + 1);
						tm rtime = *gmtime(&((time_t)r.time));
						printf("\nTime: %04u.%02u.%02u %02u:%02u", rtime.tm_year + 1900, rtime.tm_mon + 1, rtime.tm_mday, rtime.tm_hour, rtime.tm_min);
						if(precision > 0)
						{
							printf(price_format, "Open", r.open);
							printf(price_format, "High", r.high);
							printf(price_format, "Low", r.low);
							printf(price_format, "Close", r.close);
						}
						else
						{
							printf("\nOpen: %f", r.open);
							printf("\nHigh: %f", r.high);
							printf("\nLow: %f", r.low);
							printf("\nClose: %f", r.close);
						}
						printf("\nVolume: %u", r.volume);
						printf("\nSpread: %u", r.spread);
					}
					rc = HCCCloseFile(file);
					if(!CheckError(rc))
						return 1;
					printf("\n\nDone!\n");
				}
				else
					printf("\nNo rates found, halting.");
				delete[] hcc_path;
			}
			else
			{
				printf("Error: Couldn't map widechars.");
				pause();
				return 1;
			}
		}
		else
		{
			printf("Error: Couldn't allocate buffer for path.");
			pause();
			return 1;
		}
	}
	else // Show usage.
		printf("\nUsage: Drag and drop a valid HCC file to example.exe in Windows shell.\n");
	pause();
	return 0;
}

void pause()
{
	printf("\nPress any key to continue...");
	getch();
}

bool CheckError(ReturnCode rc)
{
	if(rc != HCC_SUCCESS)
	{
		wchar_t wmessage[128]; // Should fit any error message.
		wchar_t werror[160];
		if(swprintf(werror, 160, L"HCC Error (%u)", rc) > 0)
		{
			if(HCCGetErrorMessage(rc, 0, wmessage) == HCC_SUCCESS)
			{
				wcscat(werror, L": ");
				wcscat(werror, wmessage);
			}
			wprintf(L"\n%ls\n", werror);
		}
		else
			printf("\nHCC Error\n");
		pause();
		return false;
	}
	else
		return true;
}