/*

MetaTrader History Toolkit
example.mq5
v1.0
2013-03-19
Copyright  2013 Heinz Traub

*/

// This example has only been tested with the 32 bits version of MetaTrader 5.

#property description "This script loads a history file from the fixed path 'C:\Data\MyData.hcc' using the HCC Library and output some information to the Expert Advisor log."
#property description "You can modify the script to point to another path or you can create the directory, place your file there and rename it."
#property copyright "Copyright (C) 2013, Heinz Traub"
#property version "1.0"

#include "hcc.mqh"

bool CheckError(uint code)
{
	if(code != HCC_SUCCESS)
	{
		string message;
		StringInit(message, 128); // Should fit any error message.
		string err_str = "HCC Error (%u)";
		if(HCCGetErrorMessage(code, 0, message) == HCC_SUCCESS)
			StringConcatenate(err_str, err_str, ": ", message);
		PrintFormat(err_str, code);
		return false;
	}
	else
		return true;
}

void OnStart(void)
{
	Print("HCC Example Started!");
	
	uint handle = 0;
	uint result = HCCOpenFile("C:\Data\MyData.hcc", handle);
	if(!CheckError(result))
		return;
	
	string symbol;
	StringInit(symbol, 10); // Should fit any symbol.
	result = HCCGetSymbol(handle, symbol);
	if(!CheckError(result))
		return;
	else
		PrintFormat("Symbol: %s", symbol);
	
	Print("Timeframe: 1 Minute");
	
	uint hlength;
	result = HCCLoadHistory(handle, M1, 0, 3, hlength);
	if(!CheckError(result))
		return;
	else
		PrintFormat("Rates Loaded: %u", hlength);
	if(hlength > 0)
	{
		HCCRates r;
		uint precision;
		
		result = HCCGetPrecision(handle, precision);
		if(result != HCC_SUCCESS)
			precision = 5; // Should fit any symbol.
		
		for(uint i = 0; i < hlength; i++)
		{
			result = HCCGetRates(handle, i, r);
			if(!CheckError(result))
				return;
			
			PrintFormat("[RATES #%u]", i + 1);
			Print("Time: ", TimeToString(r.time));
			Print("Open: ", DoubleToString(r.open, precision));
			Print("High: ", DoubleToString(r.high, precision));
			Print("Low: ", DoubleToString(r.low, precision));
			Print("Close: ", DoubleToString(r.close, precision));
			PrintFormat("Volume: %u", r.volume);
			PrintFormat("Spread: %u", r.spread);
		}
		result = HCCCloseFile(handle);
		if(!CheckError(result))
			return;
		Print("HCC Example Done!");
	}
	else
		Print("No rates found, halting.");
}