/*

MetaTrader History Toolkit
hcc.cs
v1.0
2012-11-22
Copyright  2012 Heinz Traub

*/

// Compatible with C# 4.0 specs.

using System;
using System.Runtime.InteropServices;

// Type aliases. Scope is valid only in this source file. You can safely use native types or declare these aliases in your program.
using ReturnCode = System.UInt32;
using HCCHandle = System.UInt32;
using size_t = System.UInt32; // size_t in C# is interpreted as System.UIntPtr but in this case the size of size_t in the HCC Library is always 32 bits even if running under a 64 bits platform. This alias will be System.UIntPtr in the future when the 64 bits version of the HCC Library gets released.

static class HCC
{
	public const ReturnCode HCC_SUCCESS = 0;
	
	public enum HCC_RATES_PROPERTY : int
	{
		Time,
		Open,
		High,
		Low,
		Close,
		Volume,
		Spread,
		Period
	}
	
	public enum HCC_TIMEFRAME : int
	{
		M1 = 1, // 1 Minute.
		M2 = 2, // 2 Minutes.
		M3 = 3, // 3 Minutes.
		M4 = 4, // 4 Minutes.
		M5 = 5, // 5 Minutes.
		M6 = 6, // 6 Minutes.
		M10 = 10, // 10 Minutes.
		M12 = 12, // 12 Minutes.
		M15 = 15, // 15 Minutes.
		M20 = 20, // 20 Minutes.
		M30 = 30, // 30 Minutes.
		H1 = 60, // 1 Hour.
		H2 = 120, // 2 Hours.
		H3 = 180, // 3 Hours.
		H4 = 240, // 4 Hours.
		H6 = 360, // 6 Hours.
		H8 = 480, // 8 Hours.
		H12 = 720, // 12 Hours.
		D1 = 1440, // 1 Day.
		W1 = 10080, // 1 Week.
		MN28 = 40320, // 1 Month containing 28 days.
		MN29 = 41760, // 1 Month containing 29 days.
		MN30 = 43200, // 1 Month containing 30 days.
		MN31 = 44640, // 1 Month containing 31 days.
		MN = MN30, // Standard Month.
		TM = MN * 3, // 1 Trimester.
		SM = MN * 6, // 1 Semester.
		Y1 = 525600, // 1 Year.
		LY1 = 527040 // 1 Leap year.
	}
	
	[StructLayout(LayoutKind.Sequential, Pack = 1)]
	public struct HCCRates
	{
		#pragma warning disable 0649 // Disable field unassigned warning.
		public uint time;
		public double open;
		public double high;
		public double low;
		public double close;
		public uint volume;
		public uint spread;
		public uint period;
		#pragma warning restore 0649 // Enable field unassigned warning.
	}
	
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetErrorMessage(ReturnCode code, ushort language, [MarshalAs(UnmanagedType.LPWStr)] ref string message);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCOpenFile([MarshalAs(UnmanagedType.LPWStr)] string path, out HCCHandle file);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCCloseFile(HCCHandle file);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetSymbol(HCCHandle file, [MarshalAs(UnmanagedType.LPWStr)] ref string symbol);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetPrecision(HCCHandle file, out uint precision);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCLoadHistory(HCCHandle file, uint period, size_t start, size_t how_many, out size_t load_count);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetHistoryLength(HCCHandle file, out size_t length);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetHistoryPeriod(HCCHandle file, out uint period);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetRates(HCCHandle file, size_t index, out HCCRates rates);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetRatesDouble(HCCHandle file, size_t index, HCC_RATES_PROPERTY prop, out double value);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCGetRatesInteger(HCCHandle file, size_t index, HCC_RATES_PROPERTY prop, out uint value);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCExportHistoryCSV(HCCHandle file, [MarshalAs(UnmanagedType.LPWStr)] string csv_path, [MarshalAs(UnmanagedType.LPWStr)] string sep, [MarshalAs(UnmanagedType.LPWStr)] string decim, ushort language, size_t start, size_t how_many, out size_t export_count);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCCExportCSV(HCCHandle file, [MarshalAs(UnmanagedType.LPWStr)] string csv_path, [MarshalAs(UnmanagedType.LPWStr)] string sep, [MarshalAs(UnmanagedType.LPWStr)] string decim, ushort language, uint period, size_t start, size_t how_many, out size_t export_count);
	[DllImport("hcc.dll")]
	public static extern ReturnCode HCC2CSV([MarshalAs(UnmanagedType.LPWStr)] string hcc_path, [MarshalAs(UnmanagedType.LPWStr)] string csv_path, [MarshalAs(UnmanagedType.LPWStr)] string sep, [MarshalAs(UnmanagedType.LPWStr)] string decim, ushort language, uint period, size_t start, size_t how_many, out size_t export_count);
}